import { AbstractProperty } from "../../properties/abstract-property";
import { DependencyGraph } from "../../dependency-graph/dependency-graph";
import { AbstractPropertyWithInternals } from "../../properties/abstract-property-impl";
import { PropertyScalarBuilder } from "./property-scalar-builder";
import { TriggerBuilder } from "./trigger-builder";
import { GroupOfPropertiesBuilder } from "./group-of-properties-builder";
import { BuilderOptions } from "./builder-options";
import { AttributeId } from "../../attributes/attribute-id";
import { ListBuilder } from "./list-builder";
import { Validator } from "../../validators/validator";
import { AbstractDataProperty } from "../../properties/abstract-data-property";
import { PropertyTemplate } from "../../properties/factory/property-template";
import { RuleEngineImpl } from "../rule-engine-impl";
export declare class Builder {
    private readonly ruleEngine;
    private readonly dependencyGraph;
    private readonly propertyMap;
    private get properties();
    private readonly notEmptyIfRequiredValidator;
    private readonly defaultEmptyChoiceDisplayValue;
    private readonly defaultBackpressureConfig;
    private readonly textInterpreters;
    readonly scalar: PropertyScalarBuilder;
    readonly group: GroupOfPropertiesBuilder;
    readonly trigger: TriggerBuilder;
    readonly list: ListBuilder;
    constructor(options: BuilderOptions, ruleEngine: RuleEngineImpl, dependencyGraph: DependencyGraph, propertyMap: {
        [id: string]: AbstractPropertyWithInternals<unknown>;
    });
    private propertyScalar;
    private bindPropertyScalar;
    private groupOfProperties;
    private listOfProperties;
    private propertyList;
    private asyncPropertyList;
    private addDependencies;
    private addProperty;
    defineCustomAttribute<A>(name: string): AttributeId<A>;
    bindValidator<Properties extends readonly AbstractProperty[]>(...properties: Properties): (validator: Validator<Properties>) => void;
    decorateTemplate<T extends AbstractDataProperty<D>, D>(template: PropertyTemplate<T, D>, decorator: (property: T) => void): PropertyTemplate<T, D>;
    asVisJsData(): {
        nodes: import("../../dependency-graph/dependency-graph").VisJsNode[];
        edges: import("../../dependency-graph/dependency-graph").VisJsEdge[];
        options: {
            nodes: {
                shape: string;
                size: number;
                font: {
                    size: number;
                };
                borderWidth: number;
                shadow: boolean;
            };
            groups: {
                scalar: {
                    color: {
                        background: string;
                        border: string;
                    };
                };
                'scalar-async': {
                    color: {
                        background: string;
                        border: string;
                    };
                };
                group: {
                    color: {
                        background: string;
                        border: string;
                    };
                    shape: string;
                };
                'group-async': {
                    color: {
                        background: string;
                        border: string;
                    };
                    shape: string;
                };
                list: {
                    color: {
                        background: string;
                        border: string;
                    };
                    shape: string;
                };
                'list-async': {
                    color: {
                        background: string;
                        border: string;
                    };
                    shape: string;
                };
            };
            edges: {
                width: number;
                shadow: boolean;
            };
        };
    };
    generateNetworkGraphHtmlPage(): string;
}
