import { ListOfPropertiesTemplate, PropertyTemplate } from "../../properties/factory/property-template";
import { AbstractProperty } from "../../properties/abstract-property";
import { PropertyId } from "../../properties/property-id";
import { ListOfProperties } from "../../properties/list-of-properties";
import { ListOfPropertiesImpl } from "../../properties/list-of-properties-impl";
import { ListIndex } from "../../properties/lists/index/list-index";
import { SiblingAccess } from "../../provider/list-provider/sibling-access";
import { AbstractDataProperty } from "../../properties/abstract-data-property";
import { Rule } from "../../rules/rule";
import { BackpressureConfig } from "../../properties/backpressure/backpressure-config";
import { PropertyConfig } from "./builder-options";
import { PropertyArrayListAsyncImpl, PropertyArrayListSyncImpl } from "../../properties/property-array-list-impl";
import { AsyncListProvider, ListProvider } from "../../provider/list-provider/list-provider";
import { PropertyArrayList, PropertyArrayListCrud, PropertyArrayListCrudAsync, PropertyArrayListReadonly, PropertyArrayListReadonlyAsync } from "../../properties/property-array-list";
import { ArrayListRulesDefinition } from "../../rules/array-list-rules-definition";
import { ListOfPropertiesRulesDefinition } from "../../rules/list-of-properties-rules-definition";
export declare enum SelectionMode {
    MultiSelect = 0,
    SingleSelect = 1
}
export interface PropertyListConfig extends PropertyConfig {
    maxLength?: number;
}
export interface ComplexPropertyListConfig extends PropertyListConfig {
    selectionMode?: SelectionMode;
}
export declare class ListBuilder {
    private readonly listOfProperties;
    private readonly propertyList;
    private readonly asyncPropertyList;
    constructor(listOfProperties: <T extends AbstractDataProperty<D>, D>(id: string, itemTemplate: PropertyTemplate<T, D>, propertyConfig?: ComplexPropertyListConfig) => ListOfPropertiesImpl<T, D>, propertyList: <T>(id: PropertyId, provider: ListProvider<T>, dependencies?: readonly AbstractProperty[], propertyConfig?: PropertyListConfig, ownedProperties?: readonly AbstractProperty[]) => PropertyArrayListSyncImpl<T>, asyncPropertyList: <T>(id: PropertyId, provider: AsyncListProvider<T>, dependencies?: readonly AbstractProperty[], propertyConfig?: PropertyListConfig & {
        backpressure?: BackpressureConfig;
    }, ownedProperties?: readonly AbstractProperty[]) => PropertyArrayListAsyncImpl<T>);
    create<T extends AbstractDataProperty<D>, D>(id: PropertyId, itemTemplate: PropertyTemplate<T, D>, config?: ComplexPropertyListConfig, ...definitions: ListOfPropertiesRulesDefinition<T, D>[]): ListOfProperties<T, D>;
    template<T extends AbstractDataProperty<D>, D>(id: string, factory: (listBuilder: ListBuilder, id: PropertyId, index?: ListIndex, siblingAccess?: SiblingAccess<ListOfProperties<T, D>>) => ListOfProperties<T, D>, config?: ComplexPropertyListConfig, ...definitions: ListOfPropertiesRulesDefinition<T, D>[]): ListOfPropertiesTemplate<T, D>;
    derived: {
        sync: <Dependencies extends readonly AbstractProperty[]>(id: PropertyId, ...dependencies: Dependencies) => <T>(config: PropertyListConfig & {
            derive: Rule<[...dependencies: Dependencies], T[]>;
        }, ...definitions: ArrayListRulesDefinition<T>[]) => PropertyArrayListReadonly<T>;
        async: <Dependencies_1 extends readonly AbstractProperty[]>(id: PropertyId, ...dependencies: Dependencies_1) => <T_1>(config: PropertyListConfig & {
            derive: Rule<[...dependencies: Dependencies_1], Promise<T_1[]>>;
        }, ...definitions: ArrayListRulesDefinition<T_1>[]) => PropertyArrayListReadonlyAsync<T_1>;
    };
    crud: {
        sync: <Dependencies extends readonly AbstractProperty[]>(id: PropertyId, ...dependencies: Dependencies) => <T>(config?: (PropertyListConfig & {
            resourceProvider?: Rule<[...dependencies: Dependencies], T[]> | undefined;
        }) | undefined, ...definitions: ArrayListRulesDefinition<T>[]) => PropertyArrayListCrud<T>;
        async: <Dependencies_1 extends readonly AbstractProperty[]>(id: PropertyId, ...dependencies: Dependencies_1) => <T_1>(config: PropertyListConfig & {
            getElements: Rule<[...dependencies: Dependencies_1], Promise<T_1[]>>;
            addElement: (propertyData: T_1, index?: number | undefined) => Promise<void>;
            updateElement: (propertyData: T_1, index: number) => Promise<void>;
            removeElement: (index: number) => Promise<void>;
        }, ...definitions: ArrayListRulesDefinition<T_1>[]) => PropertyArrayListCrudAsync<T_1>;
    };
    bindListOfProperties<T extends AbstractDataProperty<D>, D>(list: ListOfProperties<T, D>, ...definitions: ListOfPropertiesRulesDefinition<T, D>[]): void;
    private bindRulesAndApplyConfigListOfProperties;
    bindPropertyArrayList<D>(list: PropertyArrayList<D>, ...definitions: ArrayListRulesDefinition<D>[]): void;
    private bindRulesAndApplyConfigArrayList;
}
