"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ListBuilder = exports.SelectionMode = void 0;
const derived_list_provider_1 = require("../../provider/list-provider/derived-list-provider");
const derived_async_list_provider_1 = require("../../provider/list-provider/derived-async-list-provider");
const crud_async_list_provider_1 = require("../../provider/list-provider/crud-async-list-provider");
const crud_list_provider_1 = require("../../provider/list-provider/crud-list-provider");
const list_of_properties_rule_builder_1 = require("./list-of-properties-rule-builder");
const property_array_list_rule_builder_1 = require("./property-array-list-rule-builder");
var SelectionMode;
(function (SelectionMode) {
    SelectionMode[SelectionMode["MultiSelect"] = 0] = "MultiSelect";
    SelectionMode[SelectionMode["SingleSelect"] = 1] = "SingleSelect";
})(SelectionMode = exports.SelectionMode || (exports.SelectionMode = {}));
class ListBuilder {
    constructor(listOfProperties, propertyList, asyncPropertyList) {
        this.listOfProperties = listOfProperties;
        this.propertyList = propertyList;
        this.asyncPropertyList = asyncPropertyList;
        this.derived = {
            sync: (id, ...dependencies) => {
                return (config, ...definitions) => {
                    const provider = new derived_list_provider_1.DerivedListProvider(dependencies, (deps) => config.derive(...deps));
                    const list = this.propertyList(id, provider, dependencies, config);
                    return this.bindRulesAndApplyConfigArrayList(list, config, ...definitions);
                };
            },
            async: (id, ...dependencies) => {
                return (config, ...definitions) => {
                    const provider = new derived_async_list_provider_1.DerivedAsyncListProvider(dependencies, (deps) => config.derive(...deps));
                    const list = this.asyncPropertyList(id, provider, dependencies, config);
                    return this.bindRulesAndApplyConfigArrayList(list, config, ...definitions);
                };
            }
        };
        this.crud = {
            sync: (id, ...dependencies) => {
                return (config, ...definitions) => {
                    if (!config) {
                        config = {};
                    }
                    if (!config.resourceProvider) {
                        config.resourceProvider = () => [];
                    }
                    const provider = new crud_list_provider_1.CrudListProvider(dependencies, (deps) => config.resourceProvider(...deps));
                    const list = this.propertyList(id, provider, dependencies, config);
                    return this.bindRulesAndApplyConfigArrayList(list, config, ...definitions);
                };
            },
            async: (id, ...dependencies) => {
                return (config, ...definitions) => {
                    const provider = new crud_async_list_provider_1.CrudAsyncListProvider(dependencies, config.addElement, (deps) => config.getElements(...deps), config.updateElement, config.removeElement);
                    const list = this.asyncPropertyList(id, provider, dependencies, config);
                    return this.bindRulesAndApplyConfigArrayList(list, config, ...definitions);
                };
            }
        };
    }
    create(id, itemTemplate, config, ...definitions) {
        const list = this.listOfProperties(id, itemTemplate, config);
        return this.bindRulesAndApplyConfigListOfProperties(list, config, ...definitions);
    }
    template(id, factory, config, ...definitions) {
        return (prefix, index, siblingAccess) => {
            const list = factory(this, `${prefix}_${id}`, index, siblingAccess);
            return this.bindRulesAndApplyConfigListOfProperties(list, config, ...definitions);
        };
    }
    bindListOfProperties(list, ...definitions) {
        const builder = new list_of_properties_rule_builder_1.ListOfPropertiesRuleBuilder(list);
        definitions.forEach(def => def.apply(builder));
    }
    bindRulesAndApplyConfigListOfProperties(prop, config, ...rulesDefintions) {
        this.bindListOfProperties(prop, ...rulesDefintions);
        if (config) {
            if (config.label !== undefined) {
                prop.defineLabel(config.label);
            }
        }
        prop.setToInitialState();
        return prop;
    }
    bindPropertyArrayList(list, ...definitions) {
        const builder = new property_array_list_rule_builder_1.PropertyArrayListRuleBuilder(list);
        definitions.forEach(def => def.apply(builder));
    }
    bindRulesAndApplyConfigArrayList(prop, config, ...rulesDefintions) {
        this.bindPropertyArrayList(prop, ...rulesDefintions);
        if (config) {
            if (config.label !== undefined) {
                prop.defineLabel(config.label);
            }
        }
        prop.setToInitialState();
        return prop;
    }
}
exports.ListBuilder = ListBuilder;
