import { PropertyId } from "../../properties/property-id";
import { PropertyScalar } from "../../properties/property-scalar";
import { PropertyScalarImpl } from "../../properties/property-scalar-impl";
import { AbstractProperty } from "../../properties/abstract-property";
import { ValueConverter } from "../../value-converter/value-converter";
import { Choice } from "../../properties/choice";
import { ValueProvider } from "../../provider/value-provider/value-provider";
import { PropertyScalarRuleBuilder } from "./property-scalar-rule-builder";
import { PropertyTemplate } from "../../properties/factory/property-template";
import { EmptyValueFcn } from "../../provider/empty-value-fcn";
import { BackpressureConfig } from "../../properties/backpressure/backpressure-config";
import { PropertyScalarWithChoices } from "../../properties/property-scalar-with-choices";
import { ListIndex } from "../../properties/lists/index/list-index";
import { SiblingAccess } from "../../provider/list-provider/sibling-access";
import { Rule } from "../../rules/rule";
import { PropertyConfig } from "./builder-options";
import { PropertyArrayListReadonly } from "../../properties/property-array-list";
import { ListBuilder } from "./list-builder";
import { ScalarRulesDefinition } from "../../rules/scalar-rules-definition";
export interface PropertyScalarConfig extends PropertyConfig {
    placeholder?: string;
    labelAndPlaceholder?: string;
}
export interface PropertyScalarValueConfig<T> extends PropertyScalarConfig {
    valueConverter?: ValueConverter<T>;
    initialValue?: T | null;
    emptyValueFcn?: EmptyValueFcn<T>;
}
export declare class PropertyScalarBuilder {
    private readonly propertyScalar;
    private readonly bindPropertyScalar;
    private readonly defaultEmptyChoiceDisplayValue;
    private readonly listBuilder;
    constructor(propertyScalar: <T>(id: PropertyId, provider: ValueProvider<T>, emptyValueFcn: EmptyValueFcn<T>, converter: ValueConverter<T>, dependencies?: readonly AbstractProperty[], propertyConfig?: {
        backpressure?: BackpressureConfig;
    }, ownedProperties?: readonly AbstractProperty[]) => PropertyScalarImpl<T>, bindPropertyScalar: <T>(prop: PropertyScalar<T>) => PropertyScalarRuleBuilder<T>, defaultEmptyChoiceDisplayValue: string | undefined, listBuilder: ListBuilder);
    template<D>(id: string, factory: (scalarBuilder: PropertyScalarBuilder, id: PropertyId, index?: ListIndex, siblingAccess?: SiblingAccess<PropertyScalar<D>>) => PropertyScalar<D>): PropertyTemplate<PropertyScalar<D>, D>;
    simpleProperty<T>(id: PropertyId, valueConverter: ValueConverter<T>, config?: PropertyScalarValueConfig<T>, ...rulesdefinitions: ScalarRulesDefinition<T>[]): PropertyScalar<T>;
    stringProperty(id: PropertyId, config?: PropertyScalarValueConfig<string>, ...rulesdefinitions: ScalarRulesDefinition<string>[]): PropertyScalar<string>;
    numberProperty(id: PropertyId, config?: PropertyScalarValueConfig<number> & {
        zeroIsConsideredAsEmpty?: boolean;
    }, ...rulesdefinitions: ScalarRulesDefinition<number>[]): PropertyScalar<number>;
    booleanProperty(id: PropertyId, config?: PropertyScalarValueConfig<boolean>, ...rulesdefinitions: ScalarRulesDefinition<boolean>[]): PropertyScalar<boolean>;
    dateProperty(id: PropertyId, config?: PropertyScalarValueConfig<Date>, ...rulesdefinitions: ScalarRulesDefinition<Date>[]): PropertyScalar<Date>;
    private defaultInitValue;
    select: {
        static: <T>(id: PropertyId, choices: Choice<T>[], config?: (PropertyScalarValueConfig<T> & {
            emptyChoice?: Choice<T> | undefined;
        }) | undefined, ...rulesdefinitions: ScalarRulesDefinition<T>[]) => PropertyScalarWithChoices<T, T>;
        derived: <Dependencies extends readonly AbstractProperty[]>(id: PropertyId, ...dependencies: Dependencies) => <T_1>(config: PropertyScalarValueConfig<T_1> & {
            derive: (...dependencies: Dependencies) => Choice<T_1>[];
            emptyChoice?: Choice<T_1> | undefined;
        }, ...rulesdefinitions: ScalarRulesDefinition<T_1>[]) => PropertyScalarWithChoices<T_1, T_1>;
        asyncDerived: <Dependencies_1 extends readonly AbstractProperty[]>(id: PropertyId, ...dependencies: Dependencies_1) => <T_2>(config: PropertyScalarValueConfig<T_2> & {
            deriveAsync: (...dependencies: Dependencies_1) => Promise<Choice<T_2>[]>;
            emptyChoice?: Choice<T_2> | undefined;
            backpressure?: BackpressureConfig | undefined;
        }, ...rulesdefinitions: ScalarRulesDefinition<T_2>[]) => PropertyScalarWithChoices<T_2, T_2>;
        withPropertySource: <T_3, S extends PropertyArrayListReadonly<Choice<T_3>>>(id: PropertyId, choicesSource: S, config?: (PropertyScalarValueConfig<T_3> & {
            emptyChoice?: Choice<T_3> | undefined;
            backpressure?: BackpressureConfig | undefined;
        }) | undefined, ...rulesdefinitions: ScalarRulesDefinition<T_3>[]) => PropertyScalarWithChoices<T_3, T_3>;
    };
    typeahead: {
        async: <T>(id: PropertyId, config: PropertyScalarValueConfig<T> & {
            fetchChoices: (currentText: string) => Promise<Choice<T>[]>;
            minimumTextLength?: number | undefined;
        }, ...rulesdefinitions: ScalarRulesDefinition<[T | null, string]>[]) => PropertyScalarWithChoices<[T | null, string], T>;
        withPropertySource: <T_1, S extends PropertyArrayListReadonly<Choice<T_1>>>(id: PropertyId, inputSource: PropertyScalar<string>, choicesSource: S, config?: PropertyScalarConfig | undefined, ...rulesdefinitions: ScalarRulesDefinition<[T_1 | null, string]>[]) => PropertyScalarWithChoices<[T_1 | null, string], T_1>;
    };
    objectProperty<T, O>(id: PropertyId, valueConverter: ValueConverter<T>, obj: O, config: PropertyScalarValueConfig<T> & {
        get: (obj: O) => T | null;
        set: (obj: O, val: T | null) => void;
        emptyValueFcn?: EmptyValueFcn<T>;
    }, ...rulesdefinitions: ScalarRulesDefinition<T>[]): PropertyScalar<T>;
    derived: {
        sync: <Dependencies extends readonly AbstractProperty[]>(id: PropertyId, ...dependencies: Dependencies) => <T>(valueConverter: ValueConverter<T>, config: PropertyScalarValueConfig<T> & {
            derive: Rule<[...dependencies: Dependencies], T | null>;
            inverse?: ((val: T | null, ...dependencies: Dependencies) => void) | undefined;
            emptyValueFcn?: EmptyValueFcn<T> | undefined;
        }, ...rulesdefinitions: ScalarRulesDefinition<T>[]) => PropertyScalar<T>;
        async: <Dependencies_1 extends readonly AbstractProperty[]>(id: PropertyId, ...dependencies: Dependencies_1) => <T_1>(valueConverter: ValueConverter<T_1>, config: PropertyScalarValueConfig<T_1> & {
            deriveAsync: Rule<[...dependencies: Dependencies_1], Promise<T_1 | null>>;
            inverseAsync?: ((val: T_1 | null, ...dependencies: Dependencies_1) => Promise<void>) | undefined;
            backpressure?: BackpressureConfig | undefined;
            emptyValueFcn?: EmptyValueFcn<T_1> | undefined;
        }, ...rulesdefinitions: ScalarRulesDefinition<T_1>[]) => PropertyScalar<T_1>;
    };
    bind<T>(prop: PropertyScalar<T>, ...rulesDefintions: ScalarRulesDefinition<T>[]): void;
    private bindRulesAndApplyConfig;
}
