import { PropertyScalar } from "../../properties/property-scalar";
import { AbstractProperty } from "../../properties/abstract-property";
import { PropertyDependencyOptions } from "../../dependency-graph/property-dependency";
import { AttributeId } from "../../attributes/attribute-id";
import { Rule } from "../../rules/rule";
import { PropertyScalarValidator } from "../../validators/single-property-validator";
import { ValidationMessage } from "../../validators/validation-message";
import { TextInterpreter, TextInterpreterFcn } from "../../util/text-interpreter/text-interpreter";
export declare class PropertyScalarRuleBuilder<T> {
    private readonly notEmptyIfRequiredValidator;
    private readonly addDependencies;
    private readonly textInterpreters;
    private readonly property;
    constructor(property: PropertyScalar<T>, notEmptyIfRequiredValidator: PropertyScalarValidator<unknown>, addDependencies: (from: readonly AbstractProperty[], to: AbstractProperty, options: PropertyDependencyOptions) => void, textInterpreters: {
        [textInterpreter in TextInterpreter]?: TextInterpreterFcn;
    });
    addValidator(validator: PropertyScalarValidator<T>): PropertyScalarRuleBuilder<T>;
    addAsyncValidator(validator: (property: PropertyScalar<T>) => Promise<ValidationMessage[] | undefined>): PropertyScalarRuleBuilder<T>;
    defineInitialValue(value: T): PropertyScalarRuleBuilder<T>;
    set<A>(attribudeId: AttributeId<A>, value: A): PropertyScalarRuleBuilder<T>;
    define<A, Dependencies extends readonly AbstractProperty[]>(attribudeId: AttributeId<A>, ...dependencies: Dependencies): (value: Rule<[self: PropertyScalar<T>, ...dependencies: Dependencies], A>) => PropertyScalarRuleBuilder<T>;
    private defineAttributeFunction;
    setVisibility(isVisible: boolean): PropertyScalarRuleBuilder<T>;
    defineVisibility<Dependencies extends readonly AbstractProperty[]>(...dependencies: Dependencies): (isVisible: Rule<[self: PropertyScalar<T>, ...dependencies: Dependencies], boolean>) => PropertyScalarRuleBuilder<T>;
    private defineVisibilityFunction;
    setRequiredIfVisible(required: boolean): PropertyScalarRuleBuilder<T>;
    defineRequiredIfVisible<Dependencies extends readonly AbstractProperty[]>(...dependencies: Dependencies): (required: Rule<[self: PropertyScalar<T>, ...dependencies: Dependencies], boolean>) => PropertyScalarRuleBuilder<T>;
    private defineRequiredIfVisibleFunction;
    defineLabelAndPlaceholder(text: string): PropertyScalarRuleBuilder<T>;
    defineLabel(text: string, textInterpreter?: TextInterpreter): PropertyScalarRuleBuilder<T>;
    definePlaceholder(placeholder: string): PropertyScalarRuleBuilder<T>;
    defineInfoText(text: string, textInterpreter?: TextInterpreter): PropertyScalarRuleBuilder<T>;
    private interpreteText;
    private checkTextInterpreterAvailable;
    onUpdated(updated: (self: PropertyScalar<T>) => void): this;
    setToInitialStateOnOtherPropertyChanged(...otherProperties: AbstractProperty[]): this;
}
