import { AbstractProperty } from "../properties/abstract-property";
import { Snapshot } from "./snapshot/snapshot";
import { Builder } from "./builder/builder";
import { PropertyGroup } from "../properties/group-of-properties";
import { RuleSet } from "./rule-set/rule-set";
import { BuilderOptions } from "./builder/builder-options";
import { AbstractDataProperty } from "../properties/abstract-data-property";
import { RuleEngineImpl } from "./rule-engine-impl";
import { PropertyId } from "../properties/property-id";
import { ValidationMessagesMap } from "../validators/validation-messages-map";
export declare function createRuleEngine(options: BuilderOptions): RuleEngineImpl;
export interface RuleEngine {
    getBuilder(): Builder;
    getPropertyById(id: PropertyId): AbstractProperty | undefined;
    validate(): Promise<ValidationMessagesMap>;
    setValidationMessages(validationMessagesMap: ValidationMessagesMap): PropertyId[];
    getValidationMessages(): ValidationMessagesMap;
    clearValidationResult(): void;
    takeSnapShot(key?: string): Snapshot;
    restoreSnapShot(key?: string): void;
    linkPropertyData<D>(propertyA: AbstractDataProperty<D>, propertyB: AbstractDataProperty<D>): void;
    unlinkPropertyData<D>(propertyA: AbstractDataProperty<D>, propertyB: AbstractDataProperty<D>): void;
    needsAnUpdate(mightHaveChanged: AbstractProperty): void;
    defineRuleSet<S extends PropertyGroup>(initFcn: (builder: Builder) => S): RuleSet<S>;
}
