import { AbstractProperty } from "./abstract-property";
import { ValueChangeListener, ValueChangeListenerReference } from "./value-change-listener";
import { Trigger } from "./trigger";
import { RuleEngineUpdateHandler } from "../engine/rule-engine-update-handler-impl";
import { ValidationMessage } from "../validators/validation-message";
import { PropertyDependency } from "../dependency-graph/property-dependency";
import { BackpressureConfig } from "./backpressure/backpressure-config";
import { ValidatorInstance } from "../engine/validation/validator-instance-impl";
import { AbstractDataProperty } from "./abstract-data-property";
import { SinglePropertyValidator } from "../validators/single-property-validator";
export interface AbstractPropertyWithInternals<D> extends AbstractDataProperty<D> {
    internallyUpdate(): Promise<void> | void;
    hasBeenUpdated(): void;
    errorWhileUpdating(error: any): void;
    dependencyHasBeenUpdated(dependency: PropertyDependency): void;
    addValidator<Properties extends readonly AbstractProperty[]>(validator: ValidatorInstance<Properties>): void;
}
export declare abstract class AbstractPropertyImpl<D> implements AbstractPropertyWithInternals<D> {
    protected updateHandler: RuleEngineUpdateHandler;
    private automaticallyUpdate;
    private manuallyTriggered?;
    private triggerListener?;
    private needsToRecompute?;
    private recomputingCount?;
    private currentRecomputing?;
    private updatedListeners?;
    private needsToRevalidate?;
    private validators?;
    private validationMessages;
    private valueChangeListeners?;
    private nextValueChangeListenerId?;
    protected singlePropertyValidators: SinglePropertyValidator<any>[];
    abstract id: string;
    backpressureConfig?: BackpressureConfig;
    private label?;
    constructor(updateHandler: RuleEngineUpdateHandler, backpressureConfig?: BackpressureConfig);
    protected abstract internallySyncUpdate(): void;
    protected abstract internallyAsyncUpdate<V>(): {
        asyncPromise: Promise<V>;
        resolve: (value: V) => void;
    };
    protected awaitAsyncUpdate(): Promise<void> | undefined;
    private updatingPromise;
    private nextRecomputingCount;
    protected syncUpdateIfNeeded(): void;
    private getTriggerListener;
    internallyUpdate(): Promise<void> | void;
    private handleBackpressure;
    private handleAsyncUpdateWithDebounceTime;
    private handleAsyncUpdate;
    needsAnUpdate(notifyOthers?: boolean, needsToRecompute?: boolean): void;
    hasBeenUpdated(): void;
    errorWhileUpdating(error: any): void;
    dependencyHasBeenUpdated(dependency: PropertyDependency): void;
    addSinglePropertyValidator(validator: SinglePropertyValidator<any>): void;
    protected getSinglePropertyValidationResults(): ValidationMessage[];
    addValidator<Properties extends readonly AbstractProperty[]>(validator: ValidatorInstance<Properties>): void;
    validate(): Promise<ValidationMessage[]>;
    isValid(): boolean;
    setValidationMessages(messages: ValidationMessage[]): void;
    getValidationMessages(): ValidationMessage[];
    clearValidationResult(): void;
    private cancelValidationAndInvalidateResults;
    private updateValidationMessages;
    registerValueChangedListener(changed: ValueChangeListener): ValueChangeListenerReference;
    deregisterValueChangedListener(changed?: ValueChangeListenerReference): void;
    protected hasValueChangeListeners(): boolean;
    protected tellValueChangeListeners(fcn: (listener: ValueChangeListener) => void): void;
    setUpdateStrategy(strategy: 'Automatic' | 'Manually'): void;
    addUpdateTrigger(trigger: Trigger, setUpdateStrategyManually?: boolean): void;
    removeUpdateTrigger(trigger: Trigger): void;
    defineLabel(label: string): void;
    getLabel(): string;
    transferData(toProperty: AbstractDataProperty<D>): void;
    abstract isAsynchronous(): boolean;
    abstract isProcessing(): boolean;
    abstract isReadOnly(): boolean;
    abstract setToInitialState(): void;
    abstract exportData(): D | null;
    abstract importData(data: D | null): void;
}
