"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ListOfPropertiesImpl = void 0;
const abstract_property_impl_1 = require("./abstract-property-impl");
const list_index_impl_1 = require("./lists/index/list-index-impl");
const single_selection_1 = require("./lists/selection/single-selection");
class ListOfPropertiesImpl extends abstract_property_impl_1.AbstractPropertyImpl {
    constructor(id, propertyTemplate, isMultiSelect, updateHandler, ownerRelation) {
        super(updateHandler);
        this.id = id;
        this.propertyTemplate = propertyTemplate;
        this.isMultiSelect = isMultiSelect;
        this.ownerRelation = ownerRelation;
        this.internalList = [];
        this.siblingCount = this.list.length;
        this.nxtId = 0;
        if (!isMultiSelect) {
            this.singleSelection = new single_selection_1.SingleSelection();
        }
    }
    get length() {
        return this.internalList.length;
    }
    get list() {
        return this.internalList.map(el => el.prop);
    }
    internallySyncUpdate() {
        throw new Error("Method not implemented.");
    }
    internallyAsyncUpdate() {
        throw new Error("Method not implemented.");
    }
    getProperty(index) {
        var _a;
        return (_a = this.internalList[index]) === null || _a === void 0 ? void 0 : _a.prop;
    }
    addProperty(property, atIndex, dontNotify) {
        const prop = this.addPropertyInternal(atIndex);
        if (property) {
            property.transferData(prop);
        }
        this.ownerRelation.addOwnerDependency(this, prop);
        if (!dontNotify) {
            this.needsAnUpdate();
        }
        return prop;
    }
    addProperties(properties, atIndex) {
        const res = [];
        const count = typeof properties === 'number' ? properties : properties.length;
        for (let c = 0; c < count; c++) {
            const property = typeof properties === 'number' ? undefined : properties[c];
            res.push(this.addProperty(property, atIndex, true));
        }
        this.needsAnUpdate();
        return res;
    }
    addPropertyData(data, atIndex) {
        const result = data.map((d, i) => {
            const index = atIndex !== undefined ? atIndex + i : undefined;
            const prop = this.addPropertyInternal(index);
            prop.importData(d);
            this.ownerRelation.addOwnerDependency(this, prop);
            return prop;
        });
        this.needsAnUpdate();
        return result;
    }
    addPropertyInternal(atIndex) {
        const propId = `${this.id}_${this.nxtId}`;
        this.nxtId++;
        const idx = atIndex !== null && atIndex !== void 0 ? atIndex : this.internalList.length;
        const index = new list_index_impl_1.ListIndexImpl(idx, this.internalList);
        const prop = this.propertyTemplate(propId, index, this);
        if (atIndex !== undefined) {
            this.internalList.splice(atIndex, 0, { prop, index });
            this.adjustIndices(atIndex + 1);
        }
        else {
            this.internalList.push({ prop, index });
        }
        return prop;
    }
    adjustIndices(start, end) {
        const to = end || this.internalList.length;
        for (let i = start !== null && start !== void 0 ? start : 0; i < to; i++) {
            this.internalList[i].index.idx = i;
        }
    }
    updatePropertyData(data, atIndex) {
        var _a;
        (_a = this.getProperty(atIndex)) === null || _a === void 0 ? void 0 : _a.importData(data);
    }
    updateProperty(property, atIndex) {
        const prop = this.getProperty(atIndex);
        if (prop) {
            property.transferData(prop);
        }
    }
    swapProperties(indexA, indexB) {
        var _a;
        if (indexA !== indexB) {
            this.movePropertyInternal(indexA, indexB);
            if (indexA < indexB) {
                this.movePropertyInternal(indexB - 1, indexA);
            }
            else {
                this.movePropertyInternal(indexB + 1, indexA);
            }
            (_a = this.singleSelection) === null || _a === void 0 ? void 0 : _a.swapProperties(indexA, indexB);
            this.needsAnUpdate();
        }
    }
    moveProperty(fromIndex, toIndex) {
        var _a;
        this.movePropertyInternal(fromIndex, toIndex);
        (_a = this.singleSelection) === null || _a === void 0 ? void 0 : _a.moveProperty(fromIndex, toIndex);
        this.needsAnUpdate();
    }
    movePropertyInternal(from, to) {
        if (from !== to) {
            const [prop] = this.internalList.splice(from, 1);
            if (prop) {
                this.internalList.splice(to, 0, prop);
                if (from < to) {
                    this.adjustIndices(from);
                }
                else {
                    this.adjustIndices(to);
                }
            }
        }
    }
    removePropertyAtIndex(index) {
        var _a;
        (_a = this.singleSelection) === null || _a === void 0 ? void 0 : _a.removePropertyAtIndex(index);
        if (index >= 0 && index < this.internalList.length) {
            const [removed] = this.internalList.splice(index, 1);
            this.adjustIndices(index);
            this.needsAnUpdate();
            return removed === null || removed === void 0 ? void 0 : removed.prop;
        }
    }
    removeProperty(property) {
        const idx = this.internalList.findIndex(el => el.prop.id === property.id);
        return idx >= 0 && this.removePropertyAtIndex(idx) !== undefined;
    }
    selectPropertyAtIndex(index) {
        if (this.internalList[index]) {
            this.internalList[index].index.isSelected = true;
        }
        if (this.singleSelection) {
            this.singleSelection.idx = index;
        }
        this.needsAnUpdate();
    }
    selectProperty(property) {
        const idx = this.internalList.findIndex(el => el.prop.id === property.id);
        this.selectPropertyAtIndex(idx);
    }
    isPropertySelectedAtIndex(index) {
        var _a, _b, _c;
        if (this.isMultiSelect) {
            return (_b = (_a = this.internalList[index]) === null || _a === void 0 ? void 0 : _a.index.isSelected) !== null && _b !== void 0 ? _b : false;
        }
        else {
            return ((_c = this.singleSelection) === null || _c === void 0 ? void 0 : _c.idx) === index;
        }
    }
    isPropertySelected(property) {
        const idx = this.internalList.findIndex(el => el.prop.id === property.id);
        return this.isPropertySelectedAtIndex(idx);
    }
    getSelectedIndices() {
        var _a, _b;
        if (this.isMultiSelect) {
            return this.internalList.filter(p => p.index.isSelected).map(p => p.index.idx);
        }
        else {
            return ((_a = this.singleSelection) === null || _a === void 0 ? void 0 : _a.idx) ? [(_b = this.singleSelection) === null || _b === void 0 ? void 0 : _b.idx] : [];
        }
    }
    getSelectedProperties() {
        if (this.isMultiSelect) {
            return this.internalList.filter(p => p.index.isSelected).map(p => ({ property: p.prop, index: p.index.idx }));
        }
        else {
            const p = this.getSelectedProperty();
            return p ? [p] : [];
        }
    }
    getSelectedProperty() {
        var _a;
        let prop;
        if (this.isMultiSelect) {
            prop = this.internalList.find(p => p.index.isSelected);
        }
        else if (((_a = this.singleSelection) === null || _a === void 0 ? void 0 : _a.idx) !== undefined) {
            prop = this.internalList[this.singleSelection.idx];
        }
        return prop && { property: prop.prop, index: prop.index.idx };
    }
    unselectPropertyAtIndex(index) {
        var _a;
        this.internalList[index].index.isSelected = false;
        if (((_a = this.singleSelection) === null || _a === void 0 ? void 0 : _a.idx) === index) {
            this.singleSelection.idx = undefined;
        }
        this.needsAnUpdate();
    }
    unselectProperty(property) {
        if (property !== undefined) {
            const idx = this.internalList.findIndex(el => el.prop.id === property.id);
            this.unselectPropertyAtIndex(idx);
        }
    }
    unselectAll() {
        var _a;
        if (this.isMultiSelect) {
            this.internalList.forEach(el => el.index.isSelected = false);
            this.needsAnUpdate();
        }
        else if ((_a = this.singleSelection) === null || _a === void 0 ? void 0 : _a.idx) {
            this.unselectPropertyAtIndex(this.singleSelection.idx);
        }
    }
    isProcessing() {
        return false;
    }
    isReadOnly() {
        return false;
    }
    isAsynchronous() {
        return false;
    }
    getSibling(atIndex) {
        return this.getProperty(atIndex);
    }
    someSibling(predicate) {
        return this.internalList.some((el) => predicate(el.prop, el.index.idx));
    }
    everySibling(predicate) {
        return this.internalList.every((el, index) => predicate(el.prop, index));
    }
    reduceSiblings(callbackfn, initialValue) {
        return this.internalList.reduce((res, el, i) => callbackfn(res, el.prop, i), initialValue);
    }
    filterSiblings(predicate) {
        return this.internalList.filter((el, i) => predicate(el.prop, i)).map(el => el.prop);
    }
    setToInitialState() {
        this.internalList = [];
    }
    exportData() {
        return this.internalList.map(prop => prop.prop.exportData());
    }
    importData(data) {
        this.setToInitialState();
        data.forEach((entry) => {
            const prop = this.addPropertyInternal();
            prop.importData(entry);
        });
        this.needsAnUpdate();
    }
}
exports.ListOfPropertiesImpl = ListOfPropertiesImpl;
