"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CrudAsyncListProvider = void 0;
class CrudAsyncListProvider {
    constructor(dependencies, create, read, update, remove) {
        this.dependencies = dependencies;
        this.create = create;
        this.read = read;
        this.update = update;
        this.remove = remove;
        this.processing = 0;
    }
    getProperties() {
        this.processing++;
        return this.read(this.dependencies).then((result) => {
            return result;
        }).finally(() => {
            this.processing--;
        });
    }
    addProperty(propertyData, index) {
        this.processing++;
        return this.create(propertyData, index).finally(() => {
            this.processing--;
        });
    }
    updateProperty(propertyData, index) {
        this.processing++;
        return this.update(propertyData, index).finally(() => {
            this.processing--;
        });
    }
    removeProperty(index) {
        this.processing++;
        return this.remove(index).finally(() => {
            this.processing--;
        });
    }
    isAsynchronous() {
        return true;
    }
    isProcessing() {
        return this.processing > 0;
    }
    shouldBeCached() {
        return false;
    }
    isReadOnly() {
        return false;
    }
}
exports.CrudAsyncListProvider = CrudAsyncListProvider;
