import { DerivedListProvider } from "../../provider/list-provider/derived-list-provider";
import { DerivedAsyncListProvider } from "../../provider/list-provider/derived-async-list-provider";
import { CrudAsyncListProvider } from "../../provider/list-provider/crud-async-list-provider";
import { CrudListProvider } from "../../provider/list-provider/crud-list-provider";
import { ListOfPropertiesRuleBuilder } from "./list-of-properties-rule-builder";
import { PropertyArrayListRuleBuilder } from "./property-array-list-rule-builder";
export var SelectionMode;
(function (SelectionMode) {
    SelectionMode[SelectionMode["MultiSelect"] = 0] = "MultiSelect";
    SelectionMode[SelectionMode["SingleSelect"] = 1] = "SingleSelect";
})(SelectionMode || (SelectionMode = {}));
export class ListBuilder {
    constructor(listOfProperties, propertyList, asyncPropertyList) {
        this.listOfProperties = listOfProperties;
        this.propertyList = propertyList;
        this.asyncPropertyList = asyncPropertyList;
        this.derived = {
            sync: (id, ...dependencies) => {
                return (config, ...definitions) => {
                    const provider = new DerivedListProvider(dependencies, (deps) => config.derive(...deps));
                    const list = this.propertyList(id, provider, dependencies, config);
                    return this.bindRulesAndApplyConfigArrayList(list, config, ...definitions);
                };
            },
            async: (id, ...dependencies) => {
                return (config, ...definitions) => {
                    const provider = new DerivedAsyncListProvider(dependencies, (deps) => config.derive(...deps));
                    const list = this.asyncPropertyList(id, provider, dependencies, config);
                    return this.bindRulesAndApplyConfigArrayList(list, config, ...definitions);
                };
            }
        };
        this.crud = {
            sync: (id, ...dependencies) => {
                return (config, ...definitions) => {
                    if (!config) {
                        config = {};
                    }
                    if (!config.resourceProvider) {
                        config.resourceProvider = () => [];
                    }
                    const provider = new CrudListProvider(dependencies, (deps) => config.resourceProvider(...deps));
                    const list = this.propertyList(id, provider, dependencies, config);
                    return this.bindRulesAndApplyConfigArrayList(list, config, ...definitions);
                };
            },
            async: (id, ...dependencies) => {
                return (config, ...definitions) => {
                    const provider = new CrudAsyncListProvider(dependencies, config.addElement, (deps) => config.getElements(...deps), config.updateElement, config.removeElement);
                    const list = this.asyncPropertyList(id, provider, dependencies, config);
                    return this.bindRulesAndApplyConfigArrayList(list, config, ...definitions);
                };
            }
        };
    }
    create(id, itemTemplate, config, ...definitions) {
        const list = this.listOfProperties(id, itemTemplate, config);
        return this.bindRulesAndApplyConfigListOfProperties(list, config, ...definitions);
    }
    template(id, factory, config, ...definitions) {
        return (prefix, index, siblingAccess) => {
            const list = factory(this, `${prefix}_${id}`, index, siblingAccess);
            return this.bindRulesAndApplyConfigListOfProperties(list, config, ...definitions);
        };
    }
    bindListOfProperties(list, ...definitions) {
        const builder = new ListOfPropertiesRuleBuilder(list);
        definitions.forEach(def => def.apply(builder));
    }
    bindRulesAndApplyConfigListOfProperties(prop, config, ...rulesDefintions) {
        this.bindListOfProperties(prop, ...rulesDefintions);
        if (config) {
            if (config.label !== undefined) {
                prop.defineLabel(config.label);
            }
        }
        prop.setToInitialState();
        return prop;
    }
    bindPropertyArrayList(list, ...definitions) {
        const builder = new PropertyArrayListRuleBuilder(list);
        definitions.forEach(def => def.apply(builder));
    }
    bindRulesAndApplyConfigArrayList(prop, config, ...rulesDefintions) {
        this.bindPropertyArrayList(prop, ...rulesDefintions);
        if (config) {
            if (config.label !== undefined) {
                prop.defineLabel(config.label);
            }
        }
        prop.setToInitialState();
        return prop;
    }
}
