import { SimpleValueProvider } from "../../provider/value-provider/simple-value-provider";
import { SelectValueConverter } from "../../value-converter/choices/select-value-converter";
import { ChoiceValueProvider } from "../../provider/value-provider/choices/choice-value-provider";
import { ObjectValueProvider } from "../../provider/value-provider/object-value-provider";
import { C } from "../../value-converter/common-value-converters";
import { DerivedValueProvider } from "../../provider/value-provider/derived-value-provider";
import { DerivedAsyncValueProvider } from "../../provider/value-provider/derived-async-value-provider";
import { EmptyValueFcns } from "../../provider/empty-value-fcn";
import { assertThat } from "../../util/assertions/assertions";
import { SelectValueProvider } from "../../provider/value-provider/choices/select-value-provider";
import { TypeaheadValueProvider } from "../../provider/value-provider/choices/typeahead-value-provider";
import { TypeaheadValueConverter } from "../../value-converter/choices/typeahead-value-converter";
import { upgradeAsPropertyWithChoices } from "../../properties/property-scalar-with-choices";
export class PropertyScalarBuilder {
    constructor(propertyScalar, bindPropertyScalar, defaultEmptyChoiceDisplayValue, listBuilder) {
        this.propertyScalar = propertyScalar;
        this.bindPropertyScalar = bindPropertyScalar;
        this.defaultEmptyChoiceDisplayValue = defaultEmptyChoiceDisplayValue;
        this.listBuilder = listBuilder;
        this.select = {
            static: (id, choices, config, ...rulesdefinitions) => {
                var _a;
                const emptyChoice = this.defaultEmptyChoiceDisplayValue !== undefined ? { value: null, displayValue: this.defaultEmptyChoiceDisplayValue } : config === null || config === void 0 ? void 0 : config.emptyChoice;
                const provider = new ChoiceValueProvider(choices, emptyChoice);
                const converter = new SelectValueConverter(choices, emptyChoice);
                const emptyValueFcn = emptyChoice ? EmptyValueFcns.choiceEmptyValueFcn(emptyChoice) : EmptyValueFcns.defaultEmptyValueFcn;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter);
                prop.defineInitialValue((emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value) !== undefined ? emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value : (_a = choices[0]) === null || _a === void 0 ? void 0 : _a.value);
                this.bindRulesAndApplyConfig(prop, config, ...rulesdefinitions);
                return upgradeAsPropertyWithChoices(prop, () => provider.getChoices());
            },
            derived: (id, ...dependencies) => (config, ...rulesdefinitions) => {
                const choicesSourceProperty = this.listBuilder.derived.sync(`${id}__choices__`, ...dependencies)({
                    derive: config.derive
                });
                return this.select.withPropertySource(id, choicesSourceProperty, config, ...rulesdefinitions);
            },
            asyncDerived: (id, ...dependencies) => (config, ...rulesdefinitions) => {
                const choicesSourceProperty = this.listBuilder.derived.async(`${id}__choices__`, ...dependencies)({
                    derive: config.deriveAsync
                });
                return this.select.withPropertySource(id, choicesSourceProperty, config, ...rulesdefinitions);
            },
            withPropertySource: (id, choicesSource, config, ...rulesdefinitions) => {
                var _a, _b;
                const emptyChoice = this.defaultEmptyChoiceDisplayValue !== undefined ? { value: null, displayValue: this.defaultEmptyChoiceDisplayValue } : config === null || config === void 0 ? void 0 : config.emptyChoice;
                const provider = new SelectValueProvider(choicesSource, emptyChoice);
                const converter = new SelectValueConverter(() => choicesSource.getElements(), emptyChoice);
                const emptyValueFcn = emptyChoice ? EmptyValueFcns.choiceEmptyValueFcn(emptyChoice) : EmptyValueFcns.defaultEmptyValueFcn;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter, [choicesSource], config, [choicesSource]);
                prop.defineInitialValue((emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value) !== undefined ? emptyChoice === null || emptyChoice === void 0 ? void 0 : emptyChoice.value : ((_b = (_a = choicesSource.getElement(0)) === null || _a === void 0 ? void 0 : _a.value) !== null && _b !== void 0 ? _b : null));
                this.bindRulesAndApplyConfig(prop, config, ...rulesdefinitions);
                return upgradeAsPropertyWithChoices(prop, () => provider.getChoices());
            }
        };
        this.typeahead = {
            async: (id, config, ...rulesdefinitions) => {
                const inputSourceProperty = this.stringProperty(`${id}__input__`);
                const choicesSourceProperty = this.listBuilder.derived.async(`${id}__choices__`, inputSourceProperty)({
                    derive: (inputProperty) => {
                        const text = inputProperty.getNonNullValue();
                        if (config.minimumTextLength && text.length < config.minimumTextLength) {
                            return Promise.resolve([]);
                        }
                        return config.fetchChoices(text);
                    }
                });
                return this.typeahead.withPropertySource(id, inputSourceProperty, choicesSourceProperty, config, ...rulesdefinitions);
            },
            withPropertySource: (id, inputSource, choicesSource, config, ...rulesdefinitions) => {
                const provider = new TypeaheadValueProvider(inputSource, choicesSource);
                const converter = new TypeaheadValueConverter();
                const emptyValueFcn = (val) => (val === null || val === void 0 ? void 0 : val[0]) == null;
                const prop = this.propertyScalar(id, provider, emptyValueFcn, converter, [inputSource], undefined, [inputSource, choicesSource]);
                this.bindRulesAndApplyConfig(prop, config, ...rulesdefinitions);
                return upgradeAsPropertyWithChoices(prop, choicesSource);
            }
        };
        this.derived = {
            sync: (id, ...dependencies) => {
                return (valueConverter, config, ...rulesdefinitions) => {
                    var _a;
                    const provider = new DerivedValueProvider(dependencies, (deps) => config.derive(...deps), config === null || config === void 0 ? void 0 : config.inverse);
                    const scalar = this.propertyScalar(id, provider, (_a = config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.defaultEmptyValueFcn, valueConverter, dependencies);
                    return this.bindRulesAndApplyConfig(scalar, config, ...rulesdefinitions);
                };
            },
            async: (id, ...dependencies) => {
                return (valueConverter, config, ...rulesdefinitions) => {
                    var _a;
                    const provider = new DerivedAsyncValueProvider(dependencies, (deps) => config.deriveAsync(...deps), config === null || config === void 0 ? void 0 : config.inverseAsync);
                    const scalar = this.propertyScalar(id, provider, (_a = config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.defaultEmptyValueFcn, valueConverter, dependencies, config);
                    return this.bindRulesAndApplyConfig(scalar, config, ...rulesdefinitions);
                };
            }
        };
    }
    template(id, factory) {
        return (prefix, index, siblingAccess) => factory(this, `${prefix}_${id}`, index, siblingAccess);
    }
    simpleProperty(id, valueConverter, config, ...rulesdefinitions) {
        var _a;
        const scalar = this.propertyScalar(id, new SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.defaultEmptyValueFcn, valueConverter);
        return this.bindRulesAndApplyConfig(scalar, config, ...rulesdefinitions);
    }
    stringProperty(id, config, ...rulesdefinitions) {
        var _a, _b;
        config = this.defaultInitValue('', config);
        const scalar = this.propertyScalar(id, new SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.defaultEmptyValueFcn, (_b = config === null || config === void 0 ? void 0 : config.valueConverter) !== null && _b !== void 0 ? _b : C.string.identity);
        return this.bindRulesAndApplyConfig(scalar, config, ...rulesdefinitions);
    }
    numberProperty(id, config, ...rulesdefinitions) {
        var _a, _b;
        assertThat(() => !(config === null || config === void 0 ? void 0 : config.zeroIsConsideredAsEmpty) || !(config === null || config === void 0 ? void 0 : config.emptyValueFcn), () => `${id}: Provide either zeroIsConsideredAsEmpty or emptyValueFcn.`);
        config = this.defaultInitValue((config === null || config === void 0 ? void 0 : config.zeroIsConsideredAsEmpty) ? 0 : null, config);
        const scalar = this.propertyScalar(id, new SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : ((config === null || config === void 0 ? void 0 : config.zeroIsConsideredAsEmpty) ? EmptyValueFcns.defaultEmptyValueFcn : EmptyValueFcns.numberEmptyValueFcn), (_b = config === null || config === void 0 ? void 0 : config.valueConverter) !== null && _b !== void 0 ? _b : C.number.default);
        return this.bindRulesAndApplyConfig(scalar, config, ...rulesdefinitions);
    }
    booleanProperty(id, config, ...rulesdefinitions) {
        var _a, _b;
        const scalar = this.propertyScalar(id, new SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.booleanEmptyValueFcn, (_b = config === null || config === void 0 ? void 0 : config.valueConverter) !== null && _b !== void 0 ? _b : C.boolean.default);
        return this.bindRulesAndApplyConfig(scalar, config, ...rulesdefinitions);
    }
    dateProperty(id, config, ...rulesdefinitions) {
        var _a, _b;
        const scalar = this.propertyScalar(id, new SimpleValueProvider(), (_a = config === null || config === void 0 ? void 0 : config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.defaultEmptyValueFcn, (_b = config === null || config === void 0 ? void 0 : config.valueConverter) !== null && _b !== void 0 ? _b : C.date.iso);
        return this.bindRulesAndApplyConfig(scalar, config, ...rulesdefinitions);
    }
    defaultInitValue(initialValue, config) {
        if (!config) {
            return {
                initialValue
            };
        }
        if (config.initialValue === undefined) {
            config.initialValue = initialValue;
        }
        return config;
    }
    objectProperty(id, valueConverter, obj, config, ...rulesdefinitions) {
        var _a;
        const scalar = this.propertyScalar(id, new ObjectValueProvider(obj, config.get, config.set), (_a = config.emptyValueFcn) !== null && _a !== void 0 ? _a : EmptyValueFcns.defaultEmptyValueFcn, valueConverter);
        return this.bindRulesAndApplyConfig(scalar, config, ...rulesdefinitions);
    }
    bind(prop, ...rulesDefintions) {
        const ruleBuilder = this.bindPropertyScalar(prop);
        rulesDefintions.forEach(def => def.apply(ruleBuilder));
    }
    bindRulesAndApplyConfig(prop, config, ...rulesDefintions) {
        this.bind(prop, ...rulesDefintions);
        if (config) {
            if (config.initialValue !== undefined) {
                prop.defineInitialValue(config.initialValue);
            }
            if (config.labelAndPlaceholder !== undefined) {
                prop.defineLabel(config.labelAndPlaceholder);
                prop.definePlaceholder(config.labelAndPlaceholder);
            }
            if (config.label !== undefined) {
                prop.defineLabel(config.label);
            }
            if (config.placeholder !== undefined) {
                prop.definePlaceholder(config.placeholder);
            }
        }
        prop.setToInitialState();
        return prop;
    }
}
