import { AbstractProperty } from "../properties/abstract-property";
import { RuleEngineUpdateHandler } from "./rule-engine-update-handler-impl";
import { Snapshot } from "./snapshot/snapshot";
import { BuilderOptions } from "./builder/builder-options";
import { Builder } from "./builder/builder";
import { ValidationResult } from "../validators/validation-result";
import { ValidatorInstance } from "./validation/validator-instance-impl";
import { AbstractDataProperty } from "../properties/abstract-data-property";
import { PropertyGroup } from "../properties/group-of-properties";
import { RuleSet } from "./rule-set/rule-set";
import { RuleEngine } from "./rule-engine";
import { ValidationMessagesMap } from "../validators/validation-messages-map";
import { PropertyId } from "../properties/property-id";
export declare class RuleEngineImpl implements RuleEngine, RuleEngineUpdateHandler {
    private readonly builder;
    private readonly propertyMap;
    private readonly dependencyGraph;
    private readonly validations;
    private readonly dataLinks;
    private readonly snapshots;
    private get properties();
    constructor(options: BuilderOptions);
    getBuilder(): Builder;
    getPropertyById(id: string): AbstractProperty | undefined;
    validate(): Promise<ValidationMessagesMap>;
    setValidationMessages(validationMessagesMap: ValidationMessagesMap): PropertyId[];
    getValidationMessages(): ValidationMessagesMap;
    clearValidationResult(): void;
    takeSnapShot(key?: string): Snapshot;
    restoreSnapShot(key?: string): void;
    linkPropertyData<D>(propertyA: AbstractDataProperty<D>, propertyB: AbstractDataProperty<D>): void;
    unlinkPropertyData<D>(propertyA: AbstractDataProperty<D>, propertyB: AbstractDataProperty<D>): void;
    private createDataLinkKey;
    needsAnUpdate(mightHaveChanged: AbstractProperty): void;
    updateValue(property: AbstractProperty): Promise<void> | undefined;
    private updateValueInternal;
    private hasBeenUpdated;
    cancelValidationAndInvalidateResults(validators: readonly ValidatorInstance<readonly AbstractProperty[]>[]): void;
    validateValidatorInstances(validators: readonly ValidatorInstance<readonly AbstractProperty[]>[]): Promise<ValidationResult | 'cancelled'>[];
    private getValidationProcess;
    defineRuleSet<S extends PropertyGroup>(initFcn: (builder: Builder) => S): RuleSet<S>;
}
