import { RuleEngineUpdateHandler } from "../engine/rule-engine-update-handler-impl";
import { AsyncListProvider, ListProvider } from "../provider/list-provider/list-provider";
import { AbstractPropertyImpl } from "./abstract-property-impl";
import { BackpressureConfig } from "./backpressure/backpressure-config";
import { PropertyArrayListCrud, PropertyArrayListCrudAsync, PropertyArrayListReadonly } from "./property-array-list";
import { PropertyId } from "./property-id";
export declare abstract class PropertyArrayListImpl<T> extends AbstractPropertyImpl<T[]> implements PropertyArrayListReadonly<T> {
    protected workingList: T[];
    setToInitialState(): void;
    get length(): number;
    abstract getElements(): T[];
    abstract getElement(atIndex: number): T | undefined;
    exportData(): T[] | null;
    importData(data: T[] | null): void;
}
export declare class PropertyArrayListSyncImpl<T> extends PropertyArrayListImpl<T> implements PropertyArrayListCrud<T> {
    readonly id: PropertyId;
    private readonly listProvider;
    constructor(id: PropertyId, listProvider: ListProvider<T>, updateHandler: RuleEngineUpdateHandler, backpressureConfig?: BackpressureConfig);
    getElements(): T[];
    getElement(atIndex: number): T | undefined;
    addElement(el: T, index?: number): void;
    updateElement(el: T, index: number): void;
    removeElement(index: number): void;
    protected internallySyncUpdate(): void;
    protected internallyAsyncUpdate(): {
        asyncPromise: Promise<any>;
        resolve: (value: any) => void;
    };
    isAsynchronous(): boolean;
    isProcessing(): boolean;
    isReadOnly(): boolean;
}
export declare class PropertyArrayListAsyncImpl<T> extends PropertyArrayListImpl<T> implements PropertyArrayListCrudAsync<T> {
    readonly id: PropertyId;
    private readonly listProvider;
    private operationPipe;
    private awaitUpdateBeforeApply?;
    private syncPromise?;
    constructor(id: PropertyId, listProvider: AsyncListProvider<T>, updateHandler: RuleEngineUpdateHandler, backpressureConfig?: BackpressureConfig);
    getElements(): T[];
    awaitElements(): Promise<T[]>;
    getElement(atIndex: number): T | undefined;
    awaitElement(atIndex: number): Promise<T | undefined>;
    addElement(el: T, index?: number): void;
    awaitAddingElement(el: T, index?: number): Promise<void>;
    updateElement(el: T, index: number): void;
    awaitUpdateElement(el: T, index: number): Promise<void>;
    removeElement(index: number): void;
    awaitRemovingElement(index: number): Promise<void>;
    private pushOperation;
    private syncList;
    private createSyncPromise;
    protected internallySyncUpdate(): void;
    protected internallyAsyncUpdate(): {
        asyncPromise: Promise<any>;
        resolve: (value: any) => void;
    };
    isAsynchronous(): boolean;
    isProcessing(): boolean;
    isReadOnly(): boolean;
    needsAnUpdate(notifyOthers?: boolean): void;
}
