var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { AbstractPropertyImpl } from "./abstract-property-impl";
import { AddOperation } from "./lists/operations/add-operation";
import { RemoveOperation } from "./lists/operations/remove-operation";
import { UpdateOperation } from "./lists/operations/update-operation";
export class PropertyArrayListImpl extends AbstractPropertyImpl {
    constructor() {
        super(...arguments);
        this.workingList = [];
    }
    setToInitialState() {
        this.needsAnUpdate(false);
    }
    get length() {
        return this.getElements().length;
    }
    exportData() {
        return this.workingList;
    }
    importData(data) {
        this.workingList = data !== null && data !== void 0 ? data : [];
    }
}
export class PropertyArrayListSyncImpl extends PropertyArrayListImpl {
    constructor(id, listProvider, updateHandler, backpressureConfig) {
        super(updateHandler, backpressureConfig);
        this.id = id;
        this.listProvider = listProvider;
    }
    getElements() {
        this.syncUpdateIfNeeded();
        return this.workingList;
    }
    getElement(atIndex) {
        this.syncUpdateIfNeeded();
        return this.workingList[atIndex];
    }
    addElement(el, index) {
        this.listProvider.addProperty(el, index);
        this.needsAnUpdate(true, false);
    }
    updateElement(el, index) {
        this.listProvider.updateProperty(el, index);
        this.needsAnUpdate(true, false);
    }
    removeElement(index) {
        this.listProvider.removeProperty(index);
        this.needsAnUpdate(true, false);
    }
    internallySyncUpdate() {
        this.workingList = this.listProvider.getProperties();
    }
    internallyAsyncUpdate() {
        return {
            asyncPromise: Promise.resolve(),
            resolve: () => ({})
        };
    }
    isAsynchronous() {
        return this.listProvider.isAsynchronous();
    }
    isProcessing() {
        return this.listProvider.isProcessing();
    }
    isReadOnly() {
        return this.listProvider.isReadOnly();
    }
}
export class PropertyArrayListAsyncImpl extends PropertyArrayListImpl {
    constructor(id, listProvider, updateHandler, backpressureConfig) {
        super(updateHandler, backpressureConfig);
        this.id = id;
        this.listProvider = listProvider;
        this.operationPipe = [];
    }
    getElements() {
        return this.workingList;
    }
    awaitElements() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.syncList();
            return this.getElements();
        });
    }
    getElement(atIndex) {
        return this.workingList[atIndex];
    }
    awaitElement(atIndex) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.syncList();
            return this.getElement(atIndex);
        });
    }
    addElement(el, index) {
        void this.awaitAddingElement(el, index);
    }
    awaitAddingElement(el, index) {
        return __awaiter(this, void 0, void 0, function* () {
            const op = new AddOperation(() => this.listProvider.addProperty(el, index), el, index);
            return this.pushOperation(op);
        });
    }
    updateElement(el, index) {
        void this.awaitUpdateElement(el, index);
    }
    awaitUpdateElement(el, index) {
        return __awaiter(this, void 0, void 0, function* () {
            const op = new UpdateOperation(() => this.listProvider.updateProperty(el, index), el, index);
            return this.pushOperation(op);
        });
    }
    removeElement(index) {
        void this.awaitRemovingElement(index);
    }
    awaitRemovingElement(index) {
        return __awaiter(this, void 0, void 0, function* () {
            const op = new RemoveOperation(() => this.listProvider.removeProperty(index), index);
            return this.pushOperation(op);
        });
    }
    pushOperation(operation) {
        const _super = Object.create(null, {
            needsAnUpdate: { get: () => super.needsAnUpdate }
        });
        return __awaiter(this, void 0, void 0, function* () {
            this.operationPipe.push(operation);
            try {
                const updated = this.awaitAsyncUpdate();
                if (updated) {
                    this.awaitUpdateBeforeApply = true;
                    yield updated;
                    if (this.awaitUpdateBeforeApply) {
                        delete this.awaitUpdateBeforeApply;
                        this.operationPipe.forEach(op => op.apply(this.workingList));
                    }
                }
                else {
                    operation.apply(this.workingList);
                }
                _super.needsAnUpdate.call(this, true, false);
            }
            catch (err) {
                delete this.awaitUpdateBeforeApply;
                this.errorWhileUpdating(err);
            }
            return this.syncList();
        });
    }
    syncList() {
        if (!this.syncPromise) {
            this.syncPromise = this.createSyncPromise().finally(() => {
                this.syncPromise = undefined;
            });
        }
        return this.syncPromise;
    }
    createSyncPromise() {
        const _super = Object.create(null, {
            needsAnUpdate: { get: () => super.needsAnUpdate }
        });
        return __awaiter(this, void 0, void 0, function* () {
            let operation;
            try {
                yield this.awaitAsyncUpdate();
                operation = this.operationPipe.shift();
                while (operation !== undefined) {
                    yield operation.synchronize();
                    operation = this.operationPipe.shift();
                }
            }
            catch (err) {
                for (let i = this.operationPipe.length - 1; i >= 0; i--) {
                    this.operationPipe[i].undo(this.workingList);
                }
                operation === null || operation === void 0 ? void 0 : operation.undo(this.workingList);
                this.operationPipe = [];
                _super.needsAnUpdate.call(this, true, false);
                this.errorWhileUpdating(err);
            }
        });
    }
    internallySyncUpdate() {
    }
    internallyAsyncUpdate() {
        this.workingList = [];
        return {
            asyncPromise: this.listProvider.getProperties(),
            resolve: (list) => {
                this.workingList = list;
            }
        };
    }
    isAsynchronous() {
        return this.listProvider.isAsynchronous();
    }
    isProcessing() {
        return this.listProvider.isProcessing() || !!this.syncPromise;
    }
    isReadOnly() {
        return this.listProvider.isReadOnly();
    }
    needsAnUpdate(notifyOthers) {
        this.operationPipe = [];
        this.workingList = [];
        super.needsAnUpdate(notifyOthers);
    }
}
